package cz.muni.fi.pb162.streams;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 *
 */
public class Main {

    public static void main(String[] args) {
        demoLambda(); // Comparator (a functional interface) defined as lambda function
        demoStreamsUppercase();
        demoStreamsSubs();
        demoStreamsPerson();
    }

    public static void demoStreamsUppercase() {
        System.out.println("*** Let's demonstrate (Java 8+) streams ***");
        System.out.println("Greetings starting with 'H' to uppercase");
        Collection<String> greetings = Stream.of("Ahoj", "Hello", "Hallo", "Szía") // creates stream from args
                .filter(s -> s.startsWith("H")) // allow elements starting with 'H'
                .map(String::toUpperCase) // convert them to uppercase
                .collect(Collectors.toList());
        System.out.println("Greetings: " + greetings);
    }

    public static void demoStreamsSubs() {
        System.out.println("*** Let's demonstrate object methods in streams ***");
        System.out.println("Greetings shortened to 4 chars");
        Stream.of("Ahojte", "Hello all", "Hallo liebe", "Szía")
                //.map(s -> s.substring(0, 4)) // trim to 4 chars
                .map(Main::trim4) // trim to 4 chars (alternative)
                .forEach(System.out::println); // printing them out
    }

    public static String trim4(final String s) {
        return s.substring(0, 4);
    }

    public static void demoStreamsPerson() {
        System.out.println("*** Let's demonstrate Person in streams ***");
        System.out.println("People without duplicates converted to string");
        Stream.of(getPeople()).parallel().distinct() // create stream without duplicates
                .map(Person::toString) // make people.toString
                .forEach(System.out::println); // printing them out
    }

    public static void demoLambda() {
        System.out.println("*** Let's demonstrate (Java 8+) lambda expressions ***");
        System.out.println("Comparator defined as functional interface using lambda expression");
        // the comparator is defined as function taking 2 params (Persons)
        // and returning int depending on order of the parameters
        // primarily age, secondly name
        final Comparator<Person> ageComparator = (Person p1, Person p2) ->
                p1.getAge() == p2.getAge() ? // same age?
                        p1.getName().compareTo(p2.getName()) : // if same ages => compare names
                        p1.getAge() - p2.getAge(); // return age difference

        final Person[] people = getPeople();
        // SortedSet peopleByAge will use ageComparator
        // Not the use of (Java 11+) type inference and new 'var' keyword instead of type
        // final is always good to put wherever it is effectively final (unmodified),
        // other languages (eg. Scala) may use 'val' instead of 'final var'
        final var peopleByAge = new TreeSet<>(ageComparator);
        // insert all people into peopleByAge
        Collections.addAll(peopleByAge, people);
        // print sorted people
        System.out.println("People sorted by age: " + peopleByAge);
    }

    private static Person[] getPeople() {
        return new Person[]{
                new Person("Tomáš", 12), new Person("Pavel", 34), new Person("Miloš", 3),
                new Person("Sabina", 11), new Person("Bára", 4), new Person("Aleš", 1),
                new Person("Kája", 11), new Person("Vlasta", 4), new Person("Jiří", 10),
                new Person("Tomáš", 12) // duplicate!
        };
    }
}
