package cz.muni.fi.pb162.streams;

/**
 * Objects of class Person represent people,
 * one object = one person having name
 *
 * For comparison: two {@link Person} objects are considered same
 * iff their names are the same => used in 'equals' method
 */
public class Person implements Comparable<Person> {

    private String name;
    private int age;

    /** Constructor: nothing new here, just initialize both attributes name */
    public Person(String name, int age) {
        this.name = name;
        this.age = age;
    }

    /** Getter for name */
    public String getName() {
        return name;
    }

    /** Getter for name */
    public int getAge() {
        return age;
    }

    /**
     * Now let us compare both name and age, only then are two people equal
     */
    @Override
    public int compareTo(Person p) {
        return name.compareTo(p.getName());
    }

    /**
     * Method equals is used to compare two objects
     * Basic principles:
     */
    @Override
    public boolean equals(Object o) {
        if(o == null || o.getClass() != Person.class) return false;
        return compareTo((Person)o) == 0;
    }

    /**
     * When you implement 'equals', you MUST always implement 'hashCode'.
     * For two objects that are equal, they must return the same hashCode.
     * Trivial solution: regardless of the object, always return the same value.
     *
     * This is not a good solution: it does not help hash-based structures
     * to efficiently store and get data, eg. in HashMap, HashSet
     */
    @Override
    public int hashCode() {
        // good solution: wherever names are equal, their hashCodes are equal.
        return name.hashCode();
        // return 0; // trivial but bad solution
    }

    @Override
    public String toString() {
        return name + " (" + age + ")";
    }
}
